@@ -32,7 +32,11 @@
 
 + (instancetype)sharedInstance;
 
-- (iTermSubpixelModel *)modelForForegoundColor:(float)foregroundComponent
-                               backgroundColor:(float)backgroundComponent;
+- (iTermSubpixelModel *)modelForForegroundColor:(float)foregroundComponent
+                                backgroundColor:(float)backgroundComponent;
+
+- (void)writeDebugDataToFolder:(NSString *)folder
+               foregroundColor:(float)foregroundComponent
+               backgroundColor:(float)backgroundComponent;
 
 @end
