@@ -1,11 +1,19 @@
 // Implements the model class for a terminal session.
 
+#import "Api.pbobjc.h"
 #import "DVR.h"
-#import "FindViewController.h"
+#import "iTermAPIHelper.h"
+#import "iTermEchoProbe.h"
+#import "iTermEncoderAdapter.h"
+#import "iTermFindDriver.h"
 #import "iTermFileDescriptorClient.h"
+#import "iTermMetalUnavailableReason.h"
 #import "iTermWeakReference.h"
 #import "ITAddressBookMgr.h"
 #import "iTermPopupWindowController.h"
+#import "iTermSessionNameController.h"
+
+#import "GPBEnumArray+iTerm.h"
 #import "LineBuffer.h"
 #import "PTYTask.h"
 #import "PTYTextView.h"
@@ -26,9 +34,24 @@ extern NSString *const kPTYSessionTmuxFontDidChange;
 // Called when captured output for the current session changes.
 extern NSString *const kPTYSessionCapturedOutputDidChange;
 
+extern NSString *const PTYSessionCreatedNotification;
+extern NSString *const PTYSessionTerminatedNotification;
+extern NSString *const PTYSessionRevivedNotification;
+extern NSString *const iTermSessionWillTerminateNotification;
+
 @class CapturedOutput;
 @class FakeWindow;
+@class iTermAction;
 @class iTermAnnouncementViewController;
+@class iTermEchoProbe;
+@class iTermExpect;
+@class iTermImageWrapper;
+@class iTermKeyBindingAction;
+@class iTermScriptHistoryEntry;
+@class iTermStatusBarViewController;
+@class iTermSwiftyStringGraph;
+@class iTermVariables;
+@class iTermVariableScope;
 @class PTYTab;
 @class PTYTask;
 @class PTYTextView;
@@ -41,14 +64,17 @@ extern NSString *const kPTYSessionCapturedOutputDidChange;
 @class iTermColorMap;
 @class iTermCommandHistoryCommandUseMO;
 @class iTermController;
-@class iTermGrowlDelegate;
+@class iTermNotificationController;
+@class iTermPromptOnCloseReason;
 @class iTermQuickLookController;
+@protocol iTermSessionScope;
 @class SessionView;
 
 typedef NS_ENUM(NSInteger, SplitSelectionMode) {
     kSplitSelectionModeOn,
     kSplitSelectionModeOff,
-    kSplitSelectionModeCancel
+    kSplitSelectionModeCancel,
+    kSplitSelectionModeInspect
 };
 
 typedef enum {
@@ -57,7 +83,7 @@ typedef enum {
     TMUX_CLIENT  // Session mirrors a tmux virtual window
 } PTYSessionTmuxMode;
 
-// This is implemented by a view that coontains a collection of sessions, nominally a tab.
+// This is implemented by a view that contains a collection of sessions, nominally a tab.
 @protocol PTYSessionDelegate<NSObject>
 
 // Return the window controller for this session. This is the "real" one, not a
@@ -72,6 +98,7 @@ typedef enum {
 // live views so it will be kept alive. Use -showLiveSession:inPlaceOf: to
 // remove a view from this list.
 - (void)addHiddenLiveView:(SessionView *)hiddenLiveView;
+- (void)session:(PTYSession *)synthetic setLiveSession:(PTYSession *)live;
 
 // Provides a tab number for the ITERM_SESSION_ID environment variable. This
 // may not correspond to the physical tab number because it's immutable for a
@@ -94,8 +121,9 @@ typedef enum {
 - (void)updateLabelAttributes;
 
 // End the session (calling terminate normally or killing/hiding a tmux
-// session), and closes the tab if neeed.
+// session), and closes the tab if needed.
 - (void)closeSession:(PTYSession *)session;
+- (void)softCloseSession:(PTYSession *)session;
 
 // Sets whether the bell indicator should show.
 - (void)setBell:(BOOL)flag;
@@ -107,14 +135,19 @@ typedef enum {
 // isn't necessarily selected.
 - (BOOL)sessionIsActiveInTab:(PTYSession *)session;
 
+// Is this session active in a currently selected tab? This ignores whether the
+// window is key.
+- (BOOL)sessionIsActiveInSelectedTab:(PTYSession *)session;
+
 // Session-initiated name change.
 - (void)nameOfSession:(PTYSession *)session didChangeTo:(NSString *)newName;
 
 // Session-initiated font size. May cause window size to adjust.
-- (void)sessionDidChangeFontSize:(PTYSession *)session;
+- (void)sessionDidChangeFontSize:(PTYSession *)session
+                    adjustWindow:(BOOL)adjustWindow;
 
 // Session-initiated resize.
-- (void)sessionInitiatedResize:(PTYSession*)session width:(int)width height:(int)height;
+- (BOOL)sessionInitiatedResize:(PTYSession*)session width:(int)width height:(int)height;
 
 // Select the "next" session in this tab.
 - (void)nextSession;
@@ -146,15 +179,10 @@ typedef enum {
            hSpacing:(double)horizontalSpacing
            vSpacing:(double)verticalSpacing;
 
-// Returns the profile to use for tmux sessions.
-- (Profile *)tmuxBookmark;
+// The tmux window title changed.
+- (void)sessionDidChangeTmuxWindowNameTo:(NSString *)newName;
 
-// Notify the tab that this session, which is a tmux gateway, received a rename of a tmux window.
-- (void)sessionWithTmuxGateway:(PTYSession *)session
-       wasNotifiedWindowWithId:(int)windowId
-                     renamedTo:(NSString *)newName;
-
-// Returns the objectSpecifier of the tab (used to identify a tab for Applescript).
+// Returns the objectSpecifier of the tab (used to identify a tab for AppleScript).
 - (NSScriptObjectSpecifier *)objectSpecifier;
 
 // Returns the tmux window ID of the containing tab. -1 if not tmux.
@@ -170,13 +198,80 @@ typedef enum {
 - (BOOL)sessionBelongsToTabWhoseSplitsAreBeingDragged;
 
 // User double clicked on title bar
-- (void)sessionDoubleClickOnTitleBar;
+- (void)sessionDoubleClickOnTitleBar:(PTYSession *)session;
 
+// Returns the 0-based pane number to use in $ITERM_SESSION_ID.
+- (NSUInteger)sessionPaneNumber:(PTYSession *)session;
+
+// The background color changed.
+- (void)sessionBackgroundColorDidChange:(PTYSession *)session;
+
+- (void)sessionKeyLabelsDidChange:(PTYSession *)session;
+
+- (void)sessionCurrentDirectoryDidChange:(PTYSession *)session;
+- (void)sessionCurrentHostDidChange:(PTYSession *)session;
+- (void)sessionProxyIconDidChange:(PTYSession *)session;
+
+// Remove a session from the tab, even if it's the only one.
+- (void)sessionRemoveSession:(PTYSession *)session;
+
+// Returns the size of the tab in rows x cols. Initial tmux client size.
+- (VT100GridSize)sessionTmuxSizeWithProfile:(Profile *)profile;
+
+// Whether metal is allowed has changed
+- (void)sessionUpdateMetalAllowed;
+- (void)sessionDidChangeMetalViewAlphaValue:(PTYSession *)session to:(CGFloat)newValue;
+
+// Amount of transparency changed (perhaps to none!)
+- (void)sessionTransparencyDidChange;
+
+// Scrollback buffer cleared.
+- (void)sessionDidClearScrollbackBuffer:(PTYSession *)session;
+
+- (iTermVariables *)sessionTabVariables;
+- (void)sessionDuplicateTab;
+
+- (BOOL)sessionShouldAutoClose:(PTYSession *)session;
+- (void)sessionDidChangeGraphic:(PTYSession *)session
+                     shouldShow:(BOOL)shouldShow
+                          image:(NSImage *)image;
+- (NSView *)sessionContainerView:(PTYSession *)session;
+
+
+- (void)sessionDraggingEntered:(PTYSession *)session;
+- (void)sessionDraggingExited:(PTYSession *)session;
+
+- (BOOL)sessionShouldSendWindowSizeIOCTL:(PTYSession *)session;
+
+- (void)sessionDidInvalidateStatusBar:(PTYSession *)session;
+- (void)sessionAddSwiftyStringsToGraph:(iTermSwiftyStringGraph *)graph;
+- (iTermVariableScope *)sessionTabScope;
+- (void)sessionDidReportSelectedTmuxPane:(PTYSession *)session;
+- (void)sessionDidUpdatePaneTitle:(PTYSession *)session;
+- (void)sessionDidSetWindowTitle:(NSString *)title;
+- (void)sessionJobDidChange:(PTYSession *)session;
+- (void)sessionEditActions;
+- (void)sessionEditSnippets;
+- (void)session:(PTYSession *)session
+setBackgroundImage:(iTermImageWrapper *)image
+           mode:(iTermBackgroundImageMode)imageMode
+backgroundColor:(NSColor *)backgroundColor;
+- (iTermImageWrapper *)sessionBackgroundImage;
+- (iTermBackgroundImageMode)sessionBackgroundImageMode;
+- (CGFloat)sessionBlend;
+- (void)sessionDidUpdatePreferencesFromProfile:(PTYSession *)session;
+- (id<iTermSwipeHandler>)sessionSwipeHandler;
+- (BOOL)sessionIsInSelectedTab:(PTYSession *)session;
+- (void)sessionDisableFocusFollowsMouseAtCurrentLocation;
 @end
 
 @class SessionView;
 @interface PTYSession : NSResponder <
-    FindViewControllerDelegate,
+    iTermEchoProbeDelegate,
+    iTermFindDriverDelegate,
+    iTermSubscribable,
+    iTermTmuxControllerSession,
+    iTermUniquelyIdentifiable,
     iTermWeaklyReferenceable,
     PopupDelegate,
     PTYTaskDelegate,
@@ -210,39 +305,34 @@ typedef enum {
 @property(nonatomic, assign) BOOL newOutput;
 
 // Do we need to prompt on close for this session?
-@property(nonatomic, readonly) BOOL promptOnClose;
+@property(nonatomic, readonly) iTermPromptOnCloseReason *promptOnCloseReason;
 
-// Array of subprocessess names.
-@property(nonatomic, readonly) NSArray *childJobNames;
+// Array of subprocessess names. WARNING: This forces a synchronous update of the process cache.
+// It is up-to-date but too slow to call frequently.
+// The first value is the name and the second is the process title.
+@property(nonatomic, readonly) NSArray<iTermTuple<NSString *, NSString *> *> *childJobNameTuples;
 
-// Is the session idle? Used by updateLabelAttributes to send a growl message when processing ends.
+// Is the session idle? Used by updateLabelAttributes to send a user notification when processing ends.
 @property(nonatomic, assign) BOOL havePostedIdleNotification;
 
-// Is there new output for the purposes of growl notifications? They run on a different schedule
+// Is there new output for the purposes of user notifications? They run on a different schedule
 // than tab colors.
 @property(nonatomic, assign) BOOL havePostedNewOutputNotification;
 
-// Session name; can be changed via escape code. The getter will add formatting to it; to retrieve
-// the value that was set, use -rawName.
-@property(nonatomic, copy) NSString *name;
+@property(nonatomic, readonly) iTermSessionNameController *nameController;
 
-// Unformatted version of -name.
-@property(nonatomic, readonly) NSString *rawName;
-
-// The original bookmark name.
-@property(nonatomic, copy) NSString *bookmarkName;
-
-// defaultName cannot be changed by the host. The getter returns a formatted name. Use
-// joblessDefaultName to get the value that was set.
-@property(nonatomic, copy) NSString *defaultName;
-
-// The value to which defaultName was last set, unadorned with additional formatting.
-@property(nonatomic, readonly) NSString *joblessDefaultName;
+// Returns the presentationName from the nameController. This is only here because
+// it's nearly impossible to find all the call sites for it.
+@property (nonatomic, readonly) NSString *name;
 
 // The window title that should be used when this session is current. Otherwise defaultName
 // should be used.
 @property(nonatomic, copy) NSString *windowTitle;
 
+// The path to the proxy icon that should be used when this session is current. If is nil the current directory icon
+// is shown.
+@property(nonatomic, retain) NSURL *preferredProxyIcon;
+
 // Shell wraps the underlying file descriptor pair.
 @property(nonatomic, retain) PTYTask *shell;
 
@@ -276,7 +366,7 @@ typedef enum {
 @property(nonatomic, assign) NSTimeInterval antiIdlePeriod;
 
 // If true, close the tab when the session ends.
-@property(nonatomic, assign) BOOL autoClose;
+@property(nonatomic, assign) iTermSessionEndAction endAction;
 
 // Should ambiguous-width characters (e.g., Greek) be treated as double-width? Usually a bad idea.
 @property(nonatomic, assign) BOOL treatAmbiguousWidthAsDoubleWidth;
@@ -284,6 +374,10 @@ typedef enum {
 // True if mouse movements are sent to the host.
 @property(nonatomic, assign) BOOL xtermMouseReporting;
 
+// True if the mouse wheel movements are sent to the host.
+@property(nonatomic, assign) BOOL xtermMouseReportingAllowMouseWheel;
+@property(nonatomic, assign) BOOL xtermMouseReportingAllowClicksAndDrags;
+
 // Profile for this session
 @property(nonatomic, copy) Profile *profile;
 
@@ -293,18 +387,21 @@ typedef enum {
 // tty device
 @property(nonatomic, readonly) NSString *tty;
 
-// True if background image should be tiled
-@property(nonatomic, assign) BOOL backgroundImageTiled;
+@property(nonatomic, assign) iTermBackgroundImageMode backgroundImageMode;
+
+// Blend level as specified in this session's profile.
+@property(nonatomic, readonly) CGFloat desiredBlend;
 
 // Filename of background image.
 @property(nonatomic, copy) NSString *backgroundImagePath;  // Used by scripting
-@property(nonatomic, retain) NSImage *backgroundImage;
+@property(nonatomic, retain) iTermImageWrapper *backgroundImage;
 
 @property(nonatomic, retain) iTermColorMap *colorMap;
 @property(nonatomic, assign) float transparency;
-@property(nonatomic, assign) float blend;
 @property(nonatomic, assign) BOOL useBoldFont;
 @property(nonatomic, assign) iTermThinStrokesSetting thinStrokes;
+@property(nonatomic, assign) BOOL asciiLigatures;
+@property(nonatomic, assign) BOOL nonAsciiLigatures;
 @property(nonatomic, assign) BOOL useItalicFont;
 
 @property(nonatomic, readonly) BOOL logging;
@@ -313,19 +410,22 @@ typedef enum {
 // Is bell currently in ringing state?
 @property(nonatomic, assign) BOOL bell;
 
-@property(nonatomic, readonly) NSDictionary *arrangement;
-
 @property(nonatomic, readonly) int columns;
 @property(nonatomic, readonly) int rows;
 
+// Returns whether this session considers itself eligible to use the Metal renderer.
+@property(nonatomic, readonly) BOOL metalAllowed;
+@property(nonatomic, readonly) BOOL idleForMetal;
+
+// Controls whether this session uses the Metal renderer.
+@property(nonatomic) BOOL useMetal;
+
 // Has this session's bookmark been divorced from the profile in the ProfileModel? Changes
-// in this bookmark may happen indepentendly of the persistent bookmark.
+// in this bookmark may happen independently of the persistent bookmark.
 // You should usually not assign to this; instead use divorceAddressBookEntryFromPreferences.
-@property(nonatomic, assign) BOOL isDivorced;
+@property(nonatomic, assign, readonly) BOOL isDivorced;
 
-@property(nonatomic, readonly) NSString *jobName;
-
-// Ignore resize notifications. This would be set because the session's size musn't be changed
+// Ignore resize notifications. This would be set because the session's size mustn't be changed
 // due to temporary changes in the window size, as code later on may need to know the session's
 // size to set the window size properly.
 @property(nonatomic, assign) BOOL ignoreResizeNotifications;
@@ -343,6 +443,9 @@ typedef enum {
 
 @property(nonatomic, retain) TmuxController *tmuxController;
 
+// Call this on tmux clients to get the session with the tmux gateway.
+@property(nonatomic, readonly) PTYSession *tmuxGatewaySession;
+
 @property(nonatomic, readonly) VT100RemoteHost *currentHost;
 
 @property(nonatomic, readonly) int tmuxPane;
@@ -352,13 +455,13 @@ typedef enum {
 @property(nonatomic, readonly) NSString *currentCommand;
 
 // Session is not in foreground and notifications are enabled on the screen.
-@property(nonatomic, readonly) BOOL shouldPostGrowlNotification;
+@property(nonatomic, readonly) BOOL shouldPostUserNotification;
 
 @property(nonatomic, readonly) BOOL hasSelection;
 
 @property(nonatomic, assign) BOOL highlightCursorLine;
 
-// Used to help remember total ordering on views while one is maximzied
+// Used to help remember total ordering on views while one is maximized
 @property(nonatomic, assign) NSPoint savedRootRelativeOrigin;
 
 // The computed label
@@ -366,13 +469,13 @@ typedef enum {
 
 // Commands issued, directories entered, and hosts connected to during this session.
 // Requires shell integration.
-@property(nonatomic, readonly) NSMutableArray *commands;  // of NSString
-@property(nonatomic, readonly) NSMutableArray *directories;  // of NSString
-@property(nonatomic, readonly) NSMutableArray *hosts;  // of VT100RemoteHost
+@property(nonatomic, readonly) NSMutableArray<NSString *> *commands;  // of NSString
+@property(nonatomic, readonly) NSMutableArray<NSString *> *directories;  // of NSString
+@property(nonatomic, readonly) NSMutableArray<VT100RemoteHost *> *hosts;  // of VT100RemoteHost
 
-// Session-defined and user-defined variables. Session-defined vars start with "session." and
-// user-defined variables start with "user.".
-@property(nonatomic, readonly) NSMutableDictionary *variables;
+@property (nonatomic, readonly) iTermVariables *variables;
+@property (nonatomic, readonly) iTermVariableScope<iTermSessionScope> *variablesScope;
+@property (nonatomic, readonly) BOOL triggerParametersUseInterpolatedStrings;
 
 @property(atomic, readonly) PTYSessionTmuxMode tmuxMode;
 
@@ -390,6 +493,17 @@ typedef enum {
 // shell integration is on). If that can't be done then the current local working directory with
 // symlinks resolved is returned.
 @property(nonatomic, readonly) NSString *currentLocalWorkingDirectory;
+// A more resilient version of the above. If the current directory cannot be determined it uses the initial directory. This allows the creation of session in succession with proper pwd recycling behavior.
+@property(nonatomic, readonly) NSString *currentLocalWorkingDirectoryOrInitialDirectory;
+
+// Async version of currentLocalWorkingDirectory.
+- (void)asyncCurrentLocalWorkingDirectory:(void (^)(NSString *pwd))completion;
+
+// Async version of currentLocalWorkingDirectoryOrInitialDirectory
+- (void)asyncCurrentLocalWorkingDirectoryOrInitialDirectory:(void (^)(NSString *pwd))completion;
+
+// Gets the local directory as URL. Weirdly, combines the remote hostname and the local path because this is really only used for the proxy icon.
+- (void)asyncGetCurrentLocationWithCompletion:(void (^)(NSURL *url))completion;
 
 // A UUID that uniquely identifies this session.
 // Used to link serialized data back to a restored session (e.g., which session
@@ -397,6 +511,7 @@ typedef enum {
 // arrangement provided to us by the OS during system window restoration with a
 // session in a saved arrangement when we're opening a saved arrangement at
 // startup instead of respecting the wishes of system window restoration.
+// Also used by the websocket API to reference a session.
 @property(nonatomic, readonly) NSString *guid;
 
 // Indicates if this session predates a tmux split pane. Used to figure out which pane is new when
@@ -404,14 +519,52 @@ typedef enum {
 @property(nonatomic, assign) BOOL sessionIsSeniorToTmuxSplitPane;
 
 @property(nonatomic, readonly) NSArray<iTermCommandHistoryCommandUseMO *> *commandUses;
+@property(nonatomic, readonly) BOOL eligibleForAutoCommandHistory;
 
 // If we want to show quicklook this will not be nil.
 @property(nonatomic, readonly) iTermQuickLookController *quickLookController;
 
+@property(nonatomic, readonly) NSDictionary<NSString *, NSString *> *keyLabels;
+@property(nonatomic, readonly) iTermRestorableSession *restorableSession;
+@property(nonatomic) BOOL copyMode;
+
+// Is the user currently typing at a password prompt?
+@property(nonatomic, readonly) BOOL passwordInput;
+
+@property(nonatomic) BOOL isSingleUseSession;
+@property(nonatomic) BOOL overrideGlobalDisableMetalWhenIdleSetting;
+@property(nonatomic, readonly) BOOL canProduceMetalFramecap;
+@property(nonatomic, readonly) NSColor *textColorForStatusBar;
+@property(nonatomic, readonly) NSColor *processedBackgroundColor;
+@property(nonatomic, readonly) NSImage *tabGraphic;
+@property(nonatomic, readonly) iTermStatusBarViewController *statusBarViewController;
+@property(nonatomic, readonly) BOOL shouldShowTabGraphic;
+@property(nonatomic, readonly) NSData *backspaceData;
+@property(nonatomic, readonly) iTermEchoProbe *echoProbe;
+@property(nonatomic, readonly) BOOL canOpenPasswordManager;
+@property(nonatomic) BOOL shortLivedSingleUse;
+@property(nonatomic, retain) NSMutableDictionary<NSString *, NSString *> *hostnameToShell;  // example.com -> fish
+@property(nonatomic, readonly) NSString *sessionId;
+@property(nonatomic, retain) NSNumber *cursorTypeOverride;
+@property(nonatomic, readonly) NSDictionary *environment;
+@property(nonatomic, readonly) BOOL hasNontrivialJob;
+@property(nonatomic, readonly) iTermExpect *expect;
+@property(nonatomic, readonly) BOOL tmuxPaused;
+@property(nonatomic, readonly) NSString *userShell;  // Something like "/bin/bash".
+
 #pragma mark - methods
 
++ (NSDictionary *)repairedArrangement:(NSDictionary *)arrangement
+             replacingProfileWithGUID:(NSString *)badGuid
+                          withProfile:(Profile *)goodProfile;
++ (NSDictionary *)repairedArrangement:(NSDictionary *)arrangement
+     replacingOldCWDOfSessionWithGUID:(NSString *)guid
+                           withOldCWD:(NSString *)replacementOldCWD;
+
 + (BOOL)handleShortcutWithoutTerminal:(NSEvent*)event;
 + (void)selectMenuItem:(NSString*)theName;
++ (void)registerBuiltInFunctions;
++ (NSMapTable<NSString *, PTYSession *> *)sessionMap;
 
 // Register the contents in the arrangement so that if the session is later
 // restored from an arrangement with the same guid as |arrangement|, the
@@ -422,6 +575,11 @@ typedef enum {
 // called after startup activities are done.
 + (void)removeAllRegisteredSessions;
 
+- (instancetype)init NS_UNAVAILABLE;
+- (instancetype)initSynthetic:(BOOL)synthetic NS_DESIGNATED_INITIALIZER;
+
+- (void)didFinishInitialization;
+
 // Jump to a particular point in time.
 - (long long)irSeekToAtLeast:(long long)timestamp;
 
@@ -431,6 +589,7 @@ typedef enum {
 // Append a bunch of lines from this (presumably synthetic) session from another (presumably live)
 // session.
 - (void)appendLinesInRange:(NSRange)rangeOfLines fromSession:(PTYSession *)source;
+- (void)appendLinesMatchingQuery:(NSString *)query fromSession:(PTYSession *)source;
 
 // Go forward/back in time. Must call setDvr:liveSession: first.
 - (void)irAdvance:(int)dir;
@@ -444,33 +603,38 @@ typedef enum {
 
 + (void)drawArrangementPreview:(NSDictionary *)arrangement frame:(NSRect)frame;
 - (void)setSizeFromArrangement:(NSDictionary*)arrangement;
-+ (PTYSession*)sessionFromArrangement:(NSDictionary*)arrangement
-                               inView:(SessionView*)sessionView
++ (PTYSession*)sessionFromArrangement:(NSDictionary *)arrangement
+                                named:(NSString *)arrangementName
+                               inView:(SessionView *)sessionView
                          withDelegate:(id<PTYSessionDelegate>)delegate
-                        forObjectType:(iTermObjectType)objectType;
+                        forObjectType:(iTermObjectType)objectType
+                   partialAttachments:(NSDictionary *)partialAttachments;
+
 + (NSDictionary *)arrangementFromTmuxParsedLayout:(NSDictionary *)parseNode
-                                         bookmark:(Profile *)bookmark;
+                                         bookmark:(Profile *)bookmark
+                                   tmuxController:(TmuxController *)tmuxController
+                                           window:(int)window;
 + (NSString *)guidInArrangement:(NSDictionary *)arrangement;
++ (NSString *)initialWorkingDirectoryFromArrangement:(NSDictionary *)arrangement;
 
 - (void)textViewFontDidChange;
 
 // Set rows, columns from arrangement.
 - (void)resizeFromArrangement:(NSDictionary *)arrangement;
 
-- (void)runCommandWithOldCwd:(NSString*)oldCWD
-               forObjectType:(iTermObjectType)objectType
-              forceUseOldCWD:(BOOL)forceUseOldCWD
-               substitutions:(NSDictionary *)substituions;
-
 - (void)startProgram:(NSString *)program
          environment:(NSDictionary *)prog_env
+         customShell:(NSString *)customShell
               isUTF8:(BOOL)isUTF8
-       substitutions:(NSDictionary *)substitutions;
+       substitutions:(NSDictionary *)substitutions
+         arrangement:(NSString *)arrangement
+          completion:(void (^)(BOOL))completion;
 
 // This is an alternative to runCommandWithOldCwd and startProgram. It attaches
 // to an existing server. Use only if [iTermAdvancedSettingsModel runJobsInServers]
 // is YES.
-- (void)attachToServer:(iTermFileDescriptorServerConnection)serverConnection;
+- (void)attachToServer:(iTermGeneralServerConnection)serverConnection
+            completion:(void (^)(void))completion;
 
 - (void)softTerminate;
 - (void)terminate;
@@ -482,6 +646,7 @@ typedef enum {
 // Preferences
 - (void)setPreferencesFromAddressBookEntry: (NSDictionary *)aePrefs;
 - (void)loadInitialColorTable;
+- (void)loadInitialColorTableAndResetCursorGuide;
 
 // Call this after the profile changed. If not divorced, the profile and
 // settings are updated. If divorced, changes are found in the session and
@@ -489,6 +654,8 @@ typedef enum {
 // overriddenFields.
 - (BOOL)reloadProfile;
 
+- (void)inheritDivorceFrom:(PTYSession *)parent decree:(NSString *)decree;
+
 - (BOOL)shouldSendEscPrefixForModifier:(unsigned int)modmask;
 
 // Writes output as though a key was pressed. Broadcast allowed. Supports tmux integration properly.
@@ -503,6 +670,11 @@ typedef enum {
                     encoding:(NSStringEncoding)optionalEncoding
                forceEncoding:(BOOL)forceEncoding;
 
+- (void)writeLatin1EncodedData:(NSData *)data broadcastAllowed:(BOOL)broadcast;
+
+- (void)updateViewBackgroundImage;
+- (void)invalidateBlend;
+
 // PTYTextView
 - (BOOL)hasTextSendingKeyMappingForEvent:(NSEvent*)event;
 - (BOOL)willHandleEvent: (NSEvent *)theEvent;
@@ -524,7 +696,7 @@ typedef enum {
 - (void)setMinimumContrast:(float)value;
 
 // Returns the frame size for a scrollview that perfectly contains the contents
-// of this session based on rows/cols, and taking into acount the presence of
+// of this session based on rows/cols, and taking into account the presence of
 // a scrollbar.
 - (NSSize)idealScrollViewSizeWithStyle:(NSScrollerStyle)scrollerStyle;
 
@@ -555,16 +727,15 @@ typedef enum {
 - (void)logStop;
 
 // Display timer stuff
-- (void)updateDisplay;
+- (void)updateDisplayBecause:(NSString *)reason;
 - (void)doAntiIdle;
 - (NSString*)ansiColorsMatchingForeground:(NSDictionary*)fg andBackground:(NSDictionary*)bg inBookmark:(Profile*)aDict;
-- (void)updateScroll;
 
 - (void)changeFontSizeDirection:(int)dir;
 - (void)setFont:(NSFont*)font
     nonAsciiFont:(NSFont*)nonAsciiFont
-    horizontalSpacing:(float)horizontalSpacing
-    verticalSpacing:(float)verticalSpacing;
+    horizontalSpacing:(CGFloat)horizontalSpacing
+    verticalSpacing:(CGFloat)verticalSpacing;
 
 // Assigns a new GUID to the session so that changes to the bookmark will not
 // affect it. Returns the GUID of a divorced bookmark. Does nothing if already
@@ -572,6 +743,10 @@ typedef enum {
 - (NSString*)divorceAddressBookEntryFromPreferences;
 - (void)remarry;
 
+// After divorcing, if there are already profile values that differ from underlying, call this to
+// set overridden fields.
+- (void)refreshOverriddenFields;
+
 // Call refresh on the textview and schedule a timer if anything is blinking.
 - (void)refresh;
 
@@ -601,16 +776,28 @@ typedef enum {
 - (void)setFocused:(BOOL)focused;
 - (BOOL)wantsContentChangedNotification;
 
-- (void)startTmuxMode;
+// The dcsID identifies the parser associated with this session. Parsers run in
+// a different thread and communication between the main thread and the parser
+// thread use the dcsID to ensure that the current parser is still the one that
+// the main thread is expecting.
+- (void)startTmuxMode:(NSString *)dcsID;
+
 - (void)tmuxDetach;
 // Two sessions are compatible if they may share the same tab. Tmux clients
 // impose this restriction because they must belong to the same controller.
 - (BOOL)isCompatibleWith:(PTYSession *)otherSession;
 - (void)setTmuxPane:(int)windowPane;
+- (void)setTmuxHistory:(NSArray<NSData *> *)history
+            altHistory:(NSArray<NSData *> *)altHistory
+                 state:(NSDictionary *)state;
+- (void)toggleTmuxPausePane;
 
 - (void)addNoteAtCursor;
-- (void)previousMarkOrNote;
-- (void)nextMarkOrNote;
+- (void)addNoteWithText:(NSString *)text inAbsoluteRange:(VT100GridAbsCoordRange)range;
+- (void)previousMark;
+- (void)nextMark;
+- (void)previousAnnotation;
+- (void)nextAnnotation;
 - (void)scrollToMark:(id<iTermMark>)mark;
 - (id<iTermMark>)markAddedAtCursorOfClass:(Class)theClass;
 
@@ -619,6 +806,7 @@ typedef enum {
 
 // Refreshes the textview and takes a snapshot of the SessionView.
 - (NSImage *)snapshot;
+- (NSImage *)snapshotCenteredOn:(VT100GridAbsCoord)coord size:(NSSize)size;
 
 - (void)enterPassword:(NSString *)password;
 
@@ -629,9 +817,16 @@ typedef enum {
 
 - (void)tryToRunShellIntegrationInstallerWithPromptCheck:(BOOL)promptCheck;
 
-- (NSDictionary *)arrangementWithContents:(BOOL)includeContents;
+- (BOOL)encodeArrangementWithContents:(BOOL)includeContents
+                              encoder:(id<iTermEncoderAdapter>)encoder;
 
 - (void)toggleTmuxZoom;
+- (void)forceTmuxDetach;
+- (void)tmuxDidDisconnect;
+
+// This is to work around a macOS bug where setNeedsDisplay: on the root view controller does not
+// cause the TextViewWrapper to be redrawn in its entirety.
+- (void)setNeedsDisplay:(BOOL)needsDisplay;
 
 // Kill the running command (if possible), print a banner, and rerun the profile's command.
 - (void)restartSession;
@@ -645,7 +840,8 @@ typedef enum {
 - (BOOL)hasAnnouncementWithIdentifier:(NSString *)identifier;
 
 // Change the current profile but keep the name the same.
-- (void)setProfile:(NSDictionary *)newProfile preservingName:(BOOL)preserveName;
+- (void)setProfile:(NSDictionary *)newProfile
+    preservingName:(BOOL)preserveName;
 
 // Make the scroll view's document view be this session's textViewWrapper.
 - (void)setScrollViewDocumentView;
@@ -653,6 +849,70 @@ typedef enum {
 // Set a value in the session's dictionary without affecting the backing profile.
 - (void)setSessionSpecificProfileValues:(NSDictionary *)newValues;
 
+- (void)useTransparencyDidChange;
+
+- (void)performKeyBindingAction:(iTermKeyBindingAction *)action event:(NSEvent *)event;
+
+- (void)setColorsFromPresetNamed:(NSString *)presetName;
+
+- (void)triggerDidDetectStartOfPromptAt:(VT100GridAbsCoord)coord;
+- (void)triggerDidDetectEndOfPromptAt:(VT100GridAbsCoord)coord;
+
+// Burys a session
+- (void)bury;
+
+// Undoes burying of a session.
+- (void)disinter;
+
+- (void)jumpToLocationWhereCurrentStatusChanged;
+- (void)updateMetalDriver;
+- (id)temporarilyDisableMetal NS_AVAILABLE_MAC(10_11);
+- (void)drawFrameAndRemoveTemporarilyDisablementOfMetalForToken:(id)token NS_AVAILABLE_MAC(10_11);
+
+- (BOOL)willEnableMetal;
+- (BOOL)metalAllowed:(out iTermMetalUnavailableReason *)reason;
+- (void)executeTokens:(const CVector *)vector bytesHandled:(int)length;
+- (void)injectData:(NSData *)data;
+
+// Call this when a session moves to a different tab or window to update the session ID.
+- (void)didMoveSession;
+- (void)triggerDidChangeNameTo:(NSString *)newName;
+- (void)didInitializeSessionWithName:(NSString *)name;
+- (void)profileNameDidChangeTo:(NSString *)name;
+- (void)profileDidChangeToProfileWithName:(NSString *)name;
+- (void)updateStatusBarStyle;
+- (BOOL)checkForCyclesInSwiftyStrings;
+- (void)applyAction:(iTermAction *)action;
+- (void)didUpdateCurrentDirectory;
+- (void)didUpdatePromptLocation;
+- (BOOL)copyModeConsumesEvent:(NSEvent *)event;
+- (Profile *)profileForSplit;
+- (void)compose;
+- (BOOL)closeComposer;
+- (void)didChangeScreen:(CGFloat)scaleFactor;
+
+#pragma mark - API
+
+- (ITMGetBufferResponse *)handleGetBufferRequest:(ITMGetBufferRequest *)request;
+- (void)handleGetPromptRequest:(ITMGetPromptRequest *)request
+                    completion:(void (^)(ITMGetPromptResponse *response))completion;
+- (void)handleListPromptsRequest:(ITMListPromptsRequest *)request
+                      completion:(void (^)(ITMListPromptsResponse *response))completion;
+- (ITMNotificationResponse *)handleAPINotificationRequest:(ITMNotificationRequest *)request
+                                            connectionKey:(NSString *)connectionKey;
+
+- (ITMSetProfilePropertyResponse_Status)handleSetProfilePropertyForAssignments:(NSArray<iTermTuple<NSString *, id> *> *)tuples
+                                                            scriptHistoryEntry:(iTermScriptHistoryEntry *)scriptHistoryEntry;
+
+- (ITMGetProfilePropertyResponse *)handleGetProfilePropertyForKeys:(NSArray<NSString *> *)keys;
+
+// Run a script-side function. Can include composition, references to variables.
+// origin is used to show why the function was called and goes in the error's title. e.g., "Trigger".
+- (void)invokeFunctionCall:(NSString *)invocation
+                     scope:(iTermVariableScope *)scope
+                    origin:(NSString *)origin;
+- (void)setParentScope:(iTermVariableScope *)parentScope;
+
 #pragma mark - Testing utilities
 
 - (void)synchronousReadTask:(NSString *)string;
