@@ -37,6 +37,7 @@
 #import "NSObject+iTerm.h"
 #import "NSSavePanel+iTerm.h"
 #import "NSURL+iTerm.h"
+#import "PasteboardHistory.h"
 #import "PTYMouseHandler.h"
 #import "PTYNoteViewController.h"
 #import "PTYTextView+Private.h"
@@ -99,7 +100,7 @@ static const NSUInteger kRectangularSelectionModifierMask = (kRectangularSelecti
     int x, y;
     int width = [self.dataSource width];
 
-    x = (locationInTextView.x - [iTermAdvancedSettingsModel terminalMargin] + self.charWidth * [iTermAdvancedSettingsModel fractionOfCharacterSelectingNextNeighbor]) / self.charWidth;
+    x = (locationInTextView.x - [iTermPreferences intForKey:kPreferenceKeySideMargins] + self.charWidth * [iTermAdvancedSettingsModel fractionOfCharacterSelectingNextNeighbor]) / self.charWidth;
     if (x < 0) {
         x = 0;
     }
@@ -138,7 +139,7 @@ static const NSUInteger kRectangularSelectionModifierMask = (kRectangularSelecti
 }
 
 - (NSPoint)pointForCoord:(VT100GridCoord)coord {
-    return NSMakePoint([iTermAdvancedSettingsModel terminalMargin] + coord.x * self.charWidth,
+    return NSMakePoint([iTermPreferences intForKey:kPreferenceKeySideMargins] + coord.x * self.charWidth,
                        coord.y * self.lineHeight);
 }
 
@@ -269,8 +270,6 @@ static const NSUInteger kRectangularSelectionModifierMask = (kRectangularSelecti
     [self updateCursor:event action:action];
 }
 
-#pragma mark - Smart Selection
-
 #pragma mark - Context Menu
 
 - (NSMenu *)menuForEvent:(NSEvent *)event {
@@ -603,7 +602,7 @@ static const NSUInteger kRectangularSelectionModifierMask = (kRectangularSelecti
 }
 
 - (NSPoint)urlActionHelper:(iTermURLActionHelper *)helper pointForCoord:(VT100GridCoord)coord {
-    NSRect windowRect = [self convertRect:NSMakeRect(coord.x * self.charWidth + [iTermAdvancedSettingsModel terminalMargin],
+    NSRect windowRect = [self convertRect:NSMakeRect(coord.x * self.charWidth + [iTermPreferences intForKey:kPreferenceKeySideMargins],
                                                      coord.y * self.lineHeight,
                                                      0,
                                                      0)
@@ -782,6 +781,16 @@ allowRightMarginOverflow:(BOOL)allowRightMarginOverflow {
     [self.delegate textViewMovePane];
 }
 
+- (void)contextMenu:(iTermTextViewContextMenuHelper *)contextMenu copyURL:(NSURL *)url {
+    NSPasteboard *pasteboard = [NSPasteboard generalPasteboard];
+    [pasteboard declareTypes:@[ NSPasteboardTypeString ]
+                       owner:self];
+    NSString *copyString = url.absoluteString;
+    [pasteboard setString:copyString
+                  forType:NSPasteboardTypeString];
+    [[PasteboardHistory sharedInstance] save:copyString];
+}
+
 - (void)contextMenuSwapSessions:(iTermTextViewContextMenuHelper *)contextMenu {
     [self.delegate textViewSwapPane];
 }
@@ -1020,7 +1029,8 @@ toggleTerminalStateForMenuItem:(nonnull NSMenuItem *)item {
 - (void)contextMenuSaveSelectionAsSnippet:(iTermTextViewContextMenuHelper *)contextMenu {
     NSString *selectedText = [self selectedText];
     iTermSnippet *snippet = [[iTermSnippet alloc] initWithTitle:selectedText
-                                                          value:selectedText];
+                                                          value:selectedText
+                                                           guid:[[NSUUID UUID] UUIDString]];
     [[iTermSnippetsModel sharedInstance] addSnippet:snippet];
 }
 
