@@ -24,6 +24,7 @@
 @class iTermExpect;
 @class iTermFindCursorView;
 @class iTermFindOnPageHelper;
+@class iTermImageWrapper;
 @class iTermQuickLookController;
 @class iTermSelection;
 @protocol iTermSemanticHistoryControllerDelegate;
@@ -63,17 +64,18 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 - (void)keyDown:(NSEvent *)event;
 - (void)keyUp:(NSEvent *)event;
 - (BOOL)hasActionableKeyMappingForEvent:(NSEvent *)event;
-- (int)optionKey;
-- (int)rightOptionKey;
+- (iTermOptionKeyBehavior)optionKey;
+- (iTermOptionKeyBehavior)rightOptionKey;
 // Contextual menu
 - (void)menuForEvent:(NSEvent *)theEvent menu:(NSMenu *)theMenu;
 - (void)pasteString:(NSString *)aString;
 - (void)paste:(id)sender;
 - (void)pasteOptions:(id)sender;
 - (void)textViewFontDidChange;
-- (void)textViewDrawBackgroundImageInView:(NSView *)view
+- (BOOL)textViewDrawBackgroundImageInView:(NSView *)view
                                  viewRect:(NSRect)rect
-                   blendDefaultBackground:(BOOL)blendDefaultBackground;
+                   blendDefaultBackground:(BOOL)blendDefaultBackground
+                            virtualOffset:(CGFloat)virtualOffset;
 - (BOOL)textViewHasBackgroundImage;
 - (void)sendEscapeSequence:(NSString *)text;
 - (void)sendHexCode:(NSString *)codes;
@@ -190,7 +192,7 @@ typedef NS_ENUM(NSInteger, PTYCharType) {
 - (void)textViewDidSelectRangeForFindOnPage:(VT100GridCoordRange)range;
 - (void)textViewNeedsDisplayInRect:(NSRect)rect;
 - (void)textViewDidSelectPasswordPrompt;
-- (NSImage *)textViewBackgroundImage;
+- (iTermImageWrapper *)textViewBackgroundImage;
 - (iTermBackgroundImageMode)backgroundImageMode;
 - (BOOL)textViewShouldDrawRect;
 - (void)textViewDidHighlightMark;
@@ -395,6 +397,9 @@ typedef void (^PTYTextViewDrawingHookBlock)(iTermTextDrawingHelper *);
 @property (nonatomic, readonly) VT100GridCoord cursorCoord;
 @property (nonatomic, readonly) iTermFindOnPageHelper *findOnPageHelper;
 
+// This is the height of the bottom margin.
+@property (nonatomic, readonly) double excess;
+
 // Returns the size of a cell for a given font. hspace and vspace are multipliers and the width
 // and height.
 + (NSSize)charSizeForFont:(NSFont*)aFont
@@ -625,6 +630,10 @@ scrollToFirstResult:(BOOL)scrollToFirstResult;
 - (id)contentWithAttributes:(BOOL)attributes;
 - (void)setUseBoldColor:(BOOL)flag brighten:(BOOL)brighten;
 
+- (void)drawRect:(NSRect)rect inView:(NSView *)view;
+
+- (void)setAlphaValue:(CGFloat)alphaValue NS_UNAVAILABLE;
+
 #pragma mark - Testing only
 
 typedef NS_ENUM(NSUInteger, iTermCopyTextStyle) {
