@@ -1,4 +1,3 @@
-// -*- mode:objc -*-
 /*
  **  ScreenChar.h
  **
@@ -30,9 +29,11 @@
  */
 
 #import <Cocoa/Cocoa.h>
+#import "ITAddressBookMgr.h"
 #import "NSStringITerm.h"
 #import "VT100GridTypes.h"
 
+@class iTermImage;
 @class iTermImageInfo;
 
 // This is used in the rightmost column when a double-width character would
@@ -66,7 +67,7 @@
 #define EOL_SOFT 1 // Soft line break (a long line was wrapped)
 #define EOL_DWC  2 // Double-width character wrapped to next line
 
-#define ONECHAR_UNKNOWN ('?')   // Relacement character for encodings other than utf-8.
+#define ONECHAR_UNKNOWN ('?')   // Replacement character for encodings other than utf-8.
 
 // Alternate semantics definitions
 // Default foreground/background color
@@ -79,6 +80,8 @@
 // foreground for background (reverse video).
 #define ALTSEM_REVERSED_DEFAULT 3
 
+#define ALTSEM_SYSTEM_MESSAGE 4
+
 typedef NS_ENUM(NSUInteger, kiTermScreenCharAnsiColor) {
     kiTermScreenCharAnsiColorBlack,
     kiTermScreenCharAnsiColorRed,
@@ -110,6 +113,10 @@ typedef enum {
     ColorModeInvalid = 3
 } ColorMode;
 
+typedef NS_ENUM(unsigned int, VT100UnderlineStyle) {
+    VT100UnderlineStyleSingle,
+    VT100UnderlineStyleCurly
+};
 
 typedef struct screen_char_t
 {
@@ -131,9 +138,9 @@ typedef struct screen_char_t
     //       magenta, cyan, and white.
     //     Values between 8 and 15 are bright versions of 0-7.
     //     Values between 16 and 255 are used for 256 color mode:
-    //       16-232: rgb value given by 16 + r*36 + g*6 + b, with each color in
+    //       16-231: rgb value given by 16 + r*36 + g*6 + b, with each color in
     //         the range [0,5].
-    //       233-255: Grayscale values from dimmest gray 233 (which is not black)
+    //       232-255: Grayscale values from dimmest gray 233 (which is not black)
     //         to brightest 255 (not white).
     // With alternate background semantics:
     //   ALTSEM_xxx (see comments above)
@@ -154,7 +161,7 @@ typedef struct screen_char_t
     unsigned int foregroundColorMode : 2;
     unsigned int backgroundColorMode : 2;
 
-    // If set, the 'code' field does not give a utf-16 value but is intead a
+    // If set, the 'code' field does not give a utf-16 value but is instead a
     // key into a string table of more complex chars (combined, surrogate pairs,
     // etc.). Valid 'code' values for a complex char are in [1, 0xefff] and will
     // be recycled as needed.
@@ -173,9 +180,16 @@ typedef struct screen_char_t
     // foregroundColor, and backgroundColor (see notes above).
     unsigned int image : 1;
 
+    unsigned int strikethrough : 1;
+    VT100UnderlineStyle underlineStyle : 1;  // VT100UnderlineStyle
+
     // These bits aren't used but are defined here so that the entire memory
     // region can be initialized.
-    unsigned int unused : 6;
+    unsigned int unused : 3;
+
+    // This comes after unused so it can be byte-aligned.
+    // If the current text is part of a hypertext link, this gives an index into the URL store.
+    unsigned short urlCode;
 } screen_char_t;
 
 // Typically used to store a single screen line.
@@ -188,6 +202,12 @@ typedef struct screen_char_t
 @property (nonatomic, assign) screen_char_t *line;  // Assume const unless instructed otherwise
 @property (nonatomic, assign) int length;
 @property (nonatomic, assign) int eol;
+@property (nonatomic) screen_char_t continuation;
+
+- (instancetype)initWithLine:(screen_char_t *)line
+                      length:(int)length
+                continuation:(screen_char_t)continuation;
+- (BOOL)isEqualToScreenCharArray:(ScreenCharArray *)other;
 @end
 
 // Standard unicode replacement string. Is a double-width character.
@@ -197,6 +217,26 @@ static inline NSString* ReplacementString()
     return [NSString stringWithCharacters:&kReplacementCharacter length:1];
 }
 
+static inline BOOL ScreenCharacterAttributesEqual(screen_char_t *c1, screen_char_t *c2) {
+    return (c1->foregroundColor == c2->foregroundColor &&
+            c1->fgGreen == c2->fgGreen &&
+            c1->fgBlue == c2->fgBlue &&
+            c1->backgroundColor == c2->backgroundColor &&
+            c1->bgGreen == c2->bgGreen &&
+            c1->bgBlue == c2->bgBlue &&
+            c1->foregroundColorMode == c2->foregroundColorMode &&
+            c1->backgroundColorMode == c2->backgroundColorMode &&
+            c1->bold == c2->bold &&
+            c1->faint == c2->faint &&
+            c1->italic == c2->italic &&
+            c1->blink == c2->blink &&
+            c1->underline == c2->underline &&
+            c1->underlineStyle == c2->underlineStyle &&
+            c1->strikethrough == c2->strikethrough &&
+            !c1->urlCode == !c2->urlCode &&  // Only tests if urlCode is zero/nonzero in both
+            c1->image == c2->image);
+}
+
 // Copy foreground color from one char to another.
 static inline void CopyForegroundColor(screen_char_t* to, const screen_char_t from)
 {
@@ -209,6 +249,9 @@ static inline void CopyForegroundColor(screen_char_t* to, const screen_char_t fr
     to->italic = from.italic;
     to->blink = from.blink;
     to->underline = from.underline;
+    to->underlineStyle = from.underlineStyle;
+    to->strikethrough = from.strikethrough;
+    to->urlCode = from.urlCode;
     to->image = from.image;
 }
 
@@ -249,7 +292,10 @@ static inline BOOL ForegroundAttributesEqual(const screen_char_t a,
         a.faint != b.faint ||
         a.italic != b.italic ||
         a.blink != b.blink ||
-        a.underline != b.underline) {
+        a.underline != b.underline ||
+        a.underlineStyle != b.underlineStyle ||
+        a.strikethrough != b.strikethrough ||
+        !a.urlCode != !b.urlCode) {
         return NO;
     }
     if (a.foregroundColorMode == b.foregroundColorMode) {
@@ -278,7 +324,10 @@ static inline BOOL ScreenCharHasDefaultAttributesAndColors(const screen_char_t s
             !s.faint &&
             !s.italic &&
             !s.blink &&
-            !s.underline);
+            !s.underline &&
+            s.underlineStyle == VT100UnderlineStyleSingle &&
+            !s.strikethrough &&
+            !s.urlCode);
 }
 
 // Represents an array of screen_char_t's as a string and facilitates mapping a
@@ -302,12 +351,16 @@ static inline BOOL ScreenCharHasDefaultAttributesAndColors(const screen_char_t s
 
 // Look up the string associated with a complex char's key.
 NSString* ComplexCharToStr(int key);
+BOOL ComplexCharCodeIsSpacingCombiningMark(unichar code);
 
 // Return a string with the contents of a screen char, which may or may not
 // be complex.
-NSString* ScreenCharToStr(screen_char_t* sct);
+NSString* ScreenCharToStr(const screen_char_t *const sct);
 NSString* CharToStr(unichar code, BOOL isComplex);
 
+// Performs the appropriate normalization.
+NSString *StringByNormalizingString(NSString *theString, iTermUnicodeNormalization normalization);
+
 // This is a faster version of ScreenCharToStr if what you want is an array of
 // unichars. Returns the number of code points appended to dest.
 int ExpandScreenChar(screen_char_t* sct, unichar* dest);
@@ -319,17 +372,6 @@ UTF32Char CharToLongChar(unichar code, BOOL isComplex);
 // returned.
 int AppendToComplexChar(int key, unichar codePoint);
 
-// Takes a non-complex character and adds a combining mark to it. It may or may not
-// become complex as a result, depending on whether there is an NFC form for the
-// new composite.
-void BeginComplexChar(screen_char_t *screenChar, unichar combiningChar, BOOL useHFSPlusMapping);
-
-// Place a complex char in a screen char.
-void SetComplexCharInScreenChar(screen_char_t *screenChar, NSString *theString, BOOL useHFSPlusMapping);
-
-// Create or lookup & return the code for a complex char.
-int GetOrSetComplexChar(NSString* str);
-
 // Translate a surrogate pair into a single utf-32 char.
 UTF32Char DecodeSurrogatePair(unichar high, unichar low);
 
@@ -357,6 +399,8 @@ int EffectiveLineLength(screen_char_t* theLine, int totalLength);
 NSString* ScreenCharArrayToStringDebug(screen_char_t* screenChars,
                                        int lineLength);
 
+NSString *DebugStringForScreenChar(screen_char_t c);
+
 // Convert an array of chars to a string, quickly.
 NSString* CharArrayToString(unichar* charHaystack, int o);
 
@@ -381,7 +425,8 @@ void StringToScreenChars(NSString *s,
                          BOOL ambiguousIsDoubleWidth,
                          int *cursorIndex,
                          BOOL *foundDwc,
-                         BOOL useHFSPlusMapping);
+                         iTermUnicodeNormalization normalization,
+                         NSInteger unicodeVersion);
 
 // Copy attributes from fg and bg, and zero out other fields. Text attributes like bold, italic, etc.
 // come from fg.
@@ -411,7 +456,7 @@ void SetPositionInImageChar(screen_char_t *charPtr, int x, int y);
 
 // Assigns an image to a code allocated by ImageCharForNewImage. data is optional and only used for
 // animated gifs.
-void SetDecodedImage(unichar code, NSImage *image, NSData *data);
+void SetDecodedImage(unichar code, iTermImage *image, NSData *data);
 
 // Releases all memory associated with an image. The code comes from ImageCharForNewImage.
 void ReleaseImage(unichar code);
@@ -425,5 +470,10 @@ VT100GridCoord GetPositionOfImageInChar(screen_char_t c);
 
 // Returns a dictionary of restorable state
 NSDictionary *ScreenCharEncodedRestorableState(void);
+NSInteger ScreenCharGeneration(void);
 void ScreenCharDecodeRestorableState(NSDictionary *state);
+void ScreenCharGarbageCollectImages(void);
+void ScreenCharClearProvisionalFlagForImageWithCode(int code);
+
+NSString *ScreenCharDescription(screen_char_t c);
 
