@@ -9,25 +9,24 @@
 #import "TaskNotifier.h"
 #import "Coprocess.h"
 #import "DebugLogging.h"
-#import "PTYTask.h"
 
 #include <sys/time.h>
 #include <sys/select.h>
 
 #define PtyTaskDebugLog(args...)
 
-NSString *const kTaskNotifierDidSpin = @"kTaskNotifierDidSpin";
+NSString *const kCoprocessStatusChangeNotification = @"kCoprocessStatusChangeNotification";
+
 static int unblockPipeR;
 static int unblockPipeW;
 
 @implementation TaskNotifier
 {
-    NSMutableArray *_tasks;
-    NSMutableArray *_coprocessOnlyTasks;
+    NSMutableArray<id<iTermTask>> *_tasks;
     // Set to true when an element of '_tasks' was modified
     BOOL tasksChanged;
-    // Protects '_tasks', '_coprocessOnlyTasks', and 'tasksChanged'.
-    NSRecursiveLock* tasksLock;
+    // Protects '_tasks' and 'tasksChanged'.
+    NSRecursiveLock *tasksLock;
 
     // A set of NSNumber*s holding pids of tasks that need to be wait()ed on
     NSMutableSet* deadpool;
@@ -51,7 +50,6 @@ static int unblockPipeW;
     if (self) {
         deadpool = [[NSMutableSet alloc] init];
         _tasks = [[NSMutableArray alloc] init];
-        _coprocessOnlyTasks = [[NSMutableArray alloc] init];
         tasksLock = [[NSRecursiveLock alloc] init];
         tasksChanged = NO;
 
@@ -73,10 +71,8 @@ static int unblockPipeW;
     return self;
 }
 
-- (void)dealloc
-{
+- (void)dealloc {
     [_tasks release];
-    [_coprocessOnlyTasks release];
     [tasksLock release];
     [deadpool release];
     close(unblockPipeR);
@@ -84,15 +80,31 @@ static int unblockPipeW;
     [super dealloc];
 }
 
-- (void)registerTask:(PTYTask*)task {
+- (void)pipeDidBreakForExternalProcessID:(pid_t)pid status:(int)status {
+    assert(pid >= 0);
+    [tasksLock lock];
+    const NSInteger i = [_tasks indexOfObjectPassingTest:^BOOL(id<iTermTask>  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
+        return obj.pid == pid;
+    }];
+    if (i == NSNotFound) {
+        [tasksLock unlock];
+        // This is the normal case — usually there'll be an error status on the FD and
+        // -deregisterTask: will take care of it.
+        return;
+    }
+    id<iTermTask> task = [[_tasks[i] retain] autorelease];
+    [_tasks removeObjectAtIndex:i];
+    tasksChanged = YES;
+    [tasksLock unlock];
+
+    [task brokenPipe];
+}
+
+- (void)registerTask:(id<iTermTask>)task {
     PtyTaskDebugLog(@"registerTask: lock\n");
     [tasksLock lock];
     PtyTaskDebugLog(@"Add task at %p\n", (void*)task);
-    if (task.isCoprocessOnly) {
-        [_coprocessOnlyTasks addObject:task];
-    } else {
-        [_tasks addObject:task];
-    }
+    [_tasks addObject:task];
     PtyTaskDebugLog(@"There are now %lu tasks\n", (unsigned long)_tasks.count);
     tasksChanged = YES;
     PtyTaskDebugLog(@"registerTask: unlock\n");
@@ -100,29 +112,23 @@ static int unblockPipeW;
     [self unblock];
 }
 
-- (void)deregisterTask:(PTYTask *)task {
+- (void)deregisterTask:(id<iTermTask>)task {
     PtyTaskDebugLog(@"deregisterTask: lock\n");
     [tasksLock lock];
     PtyTaskDebugLog(@"Begin remove task %p\n", (void*)task);
     PtyTaskDebugLog(@"Add %d to deadpool", [task pid]);
-    pid_t pid = task.pid;
-    if (pid != -1 && task.pidIsChild) {
-        // Not a task running in a server.
-        [deadpool addObject:@([task pid])];
-    } else if (task.serverPid != -1 && !task.pidIsChild) {
-        // Prevent server from becoming a zombie.
-        [deadpool addObject:@(task.serverPid)];
+    pid_t pidToWaitOn = task.pidToWaitOn;
+    if (pidToWaitOn > 0) {
+        [deadpool addObject:@(pidToWaitOn)];
     }
     if ([task hasCoprocess]) {
         [deadpool addObject:@([[task coprocess] pid])];
     }
     [_tasks removeObject:task];
-    [_coprocessOnlyTasks removeObject:task];
     tasksChanged = YES;
-    PtyTaskDebugLog(@"End remove task %p. There are now %lu tasks and %ld coprocess-only tasks.\n",
-                    (void*)task,
-                    (unsigned long)[_tasks count],
-                    (unsigned long)_coprocessOnlyTasks.count);
+    PtyTaskDebugLog(@"End remove task %p. There are now %lu tasks.\n",
+                    (void *)task,
+                    (unsigned long)[_tasks count]);
     PtyTaskDebugLog(@"deregisterTask: unlock\n");
     [tasksLock unlock];
     [self unblock];
@@ -136,8 +142,7 @@ static int unblockPipeW;
     [self unblock];
 }
 
-- (void)unblock
-{
+- (void)unblock {
     UnblockTaskNotifier();
 }
 
@@ -148,7 +153,7 @@ void UnblockTaskNotifier(void) {
     write(unblockPipeW, &dummy, 1);
 }
 
-- (BOOL)handleReadOnFileDescriptor:(int)fd task:(PTYTask *)task fdSet:(fd_set *)fdSet {
+- (BOOL)handleReadOnFileDescriptor:(int)fd task:(id<iTermTask>)task fdSet:(fd_set *)fdSet {
     if (FD_ISSET(fd, fdSet)) {
         PtyTaskDebugLog(@"run/processRead: unlock");
         [tasksLock unlock];
@@ -164,11 +169,13 @@ void UnblockTaskNotifier(void) {
     return NO;
 }
 
-- (BOOL)handleWriteOnFileDescriptor:(int)fd task:(PTYTask *)task fdSet:(fd_set *)fdSet {
+- (BOOL)handleWriteOnFileDescriptor:(int)fd task:(id<iTermTask>)task fdSet:(fd_set *)fdSet {
     if (FD_ISSET(fd, fdSet)) {
         PtyTaskDebugLog(@"run/processWrite: unlock");
         [tasksLock unlock];
+        [task retain];
         [task processWrite];
+        [task release];
         PtyTaskDebugLog(@"run/processWrite: lock");
         [tasksLock lock];
         if (tasksChanged) {
@@ -180,7 +187,7 @@ void UnblockTaskNotifier(void) {
     return NO;
 }
 
-- (BOOL)handleErrorOnFileDescriptor:(int)fd task:(PTYTask *)task fdSet:(fd_set *)fdSet {
+- (BOOL)handleErrorOnFileDescriptor:(int)fd task:(id<iTermTask>)task fdSet:(fd_set *)fdSet {
     if (FD_ISSET(fd, fdSet)) {
         PtyTaskDebugLog(@"run/brokenPipe: unlock");
         [tasksLock unlock];
@@ -199,7 +206,7 @@ void UnblockTaskNotifier(void) {
 }
 
 - (void)handleReadOnFileDescriptor:(int)fd
-                              task:(PTYTask *)task
+                              task:(id<iTermTask>)task
                      withCoprocess:(Coprocess *)coprocess
                              fdSet:(fd_set *)fdSet {
     if (![coprocess eof] && FD_ISSET(fd, fdSet)) {
@@ -230,14 +237,13 @@ void UnblockTaskNotifier(void) {
     }
 }
 
-- (void)run
-{
+- (void)run {
     fd_set rfds;
     fd_set wfds;
     fd_set efds;
     int highfd;
-    NSEnumerator* iter;
-    NSAutoreleasePool* autoreleasePool = [[NSAutoreleasePool alloc] init];
+    NSEnumerator *iter;
+    NSAutoreleasePool *autoreleasePool = [[NSAutoreleasePool alloc] init];
 
     // FIXME: replace this with something better...
     for(;;) {
@@ -249,26 +255,22 @@ void UnblockTaskNotifier(void) {
         // Unblock pipe to interrupt select() whenever a PTYTask register/unregisters
         highfd = unblockPipeR;
         FD_SET(unblockPipeR, &rfds);
-        NSMutableSet* handledFds = [[NSMutableSet alloc] initWithCapacity:256];
+        NSMutableSet *handledFds = [[NSMutableSet alloc] initWithCapacity:256];
 
         // Add all the PTYTask pipes
         PtyTaskDebugLog(@"run1: lock");
         [tasksLock lock];
         PtyTaskDebugLog(@"Begin cleaning out dead tasks");
-        for (PTYTask *theTask in _tasks) {
+        NSMutableArray<id<iTermTask>> *tasksToDeregister = [NSMutableArray array];
+        for (id<iTermTask> theTask in _tasks) {
             if ([theTask fd] < 0) {
                 PtyTaskDebugLog(@"Deregister dead task %@\n", theTask);
-                [self deregisterTask:theTask];
+                [tasksToDeregister addObject:theTask];
             }
         }
-        // Make a copy because -deregisterTask modifies _coprocessOnlyTasks
-        NSArray *coprocessOnlyTasks = [_coprocessOnlyTasks copy];
-        for (PTYTask *theTask in coprocessOnlyTasks) {
-            if ([theTask coprocessOnlyTaskIsDead]) {
-                [self deregisterTask:theTask];
-            }
+        for (id<iTermTask> theTask in tasksToDeregister) {
+            [self deregisterTask:theTask];
         }
-        [coprocessOnlyTasks release];
 
         if ([deadpool count] > 0) {
             // waitpid() on pids that we think are dead or will be dead soon.
@@ -298,58 +300,58 @@ void UnblockTaskNotifier(void) {
 
         // Figure out the file descriptors to select on.
         PtyTaskDebugLog(@"Begin enumeration over %lu tasks\n", (unsigned long)[_tasks count]);
-        for (NSArray *taskArray in @[ _tasks, _coprocessOnlyTasks ]) {
-            for (PTYTask *task in taskArray) {
-                PtyTaskDebugLog(@"Got task %@\n", task);
-                int fd = [task fd];
-                if (fd < 0) {
-                    PtyTaskDebugLog(@"Task has fd of %d\n", fd);
-                } else {
-                    // PtyTaskDebugLog(@"Select on fd %d\n", fd);
-                    if (fd > highfd) {
-                        highfd = fd;
-                    }
-                    if ([task wantsRead]) {
-                        FD_SET(fd, &rfds);
-                    }
-                    if ([task wantsWrite]) {
-                        FD_SET(fd, &wfds);
-                    }
-                    FD_SET(fd, &efds);
+        for (id<iTermTask> task in _tasks) {
+            PtyTaskDebugLog(@"Got task %@\n", task);
+            int fd = [task fd];
+            if (fd < 0) {
+                PtyTaskDebugLog(@"Task has fd of %d\n", fd);
+            } else {
+                // PtyTaskDebugLog(@"Select on fd %d\n", fd);
+                if (fd > highfd) {
+                    highfd = fd;
                 }
+                if ([task wantsRead]) {
+                    FD_SET(fd, &rfds);
+                }
+                if ([task wantsWrite]) {
+                    FD_SET(fd, &wfds);
+                }
+                FD_SET(fd, &efds);
+            }
 
-                @synchronized (task) {
-                    Coprocess *coprocess = [task coprocess];
-                    if (coprocess) {
-                        if ([coprocess wantToRead] && [task writeBufferHasRoom]) {
-                            int rfd = [coprocess readFileDescriptor];
-                            if (rfd > highfd) {
-                                highfd = rfd;
-                            }
-                            FD_SET(rfd, &rfds);
+            @synchronized (task) {
+                Coprocess *coprocess = [task coprocess];
+                if (coprocess) {
+                    if ([coprocess wantToRead] && [task writeBufferHasRoom]) {
+                        int rfd = [coprocess readFileDescriptor];
+                        if (rfd > highfd) {
+                            highfd = rfd;
                         }
-                        if ([coprocess wantToWrite]) {
-                            int wfd = [coprocess writeFileDescriptor];
-                            if (wfd > highfd) {
-                                highfd = wfd;
-                            }
-                            FD_SET(wfd, &wfds);
+                        FD_SET(rfd, &rfds);
+                    }
+                    if ([coprocess wantToWrite]) {
+                        int wfd = [coprocess writeFileDescriptor];
+                        if (wfd > highfd) {
+                            highfd = wfd;
                         }
-                        if (![coprocess eof]) {
-                            int rfd = [coprocess readFileDescriptor];
-                            if (rfd > highfd) {
-                                highfd = rfd;
-                            }
-                            FD_SET(rfd, &efds);
+                        FD_SET(wfd, &wfds);
+                    }
+                    if (![coprocess eof]) {
+                        int rfd = [coprocess readFileDescriptor];
+                        if (rfd > highfd) {
+                            highfd = rfd;
                         }
+                        FD_SET(rfd, &efds);
                     }
                 }
             }
         }
+
         PtyTaskDebugLog(@"run1: unlock");
         [tasksLock unlock];
 
-        [[NSNotificationCenter defaultCenter] postNotificationName:kTaskNotifierDidSpin object:nil];
+        [autoreleasePool drain];
+        autoreleasePool = [[NSAutoreleasePool alloc] init];
 
         // Poll...
         if (select(highfd+1, &rfds, &wfds, &efds, NULL) <= 0) {
@@ -377,7 +379,7 @@ void UnblockTaskNotifier(void) {
         iter = [_tasks objectEnumerator];
         BOOL notifyOfCoprocessChange = NO;
 
-        PTYTask *task;
+        id<iTermTask> task;
         while ((task = [iter nextObject])) {
             PtyTaskDebugLog(@"Got task %@\n", task);
             int fd = [task fd];
@@ -440,42 +442,6 @@ void UnblockTaskNotifier(void) {
             }
         }
 
-        // Handle coprocess reads, writes, and deaths for coprocess-only tasks.
-          PtyTaskDebugLog(@"Iterating over %lu coprocess-only tasks\n", (unsigned long)_coprocessOnlyTasks.count);
-          for (PTYTask *coprocessOnlyTask in _coprocessOnlyTasks) {
-            @synchronized (coprocessOnlyTask) {
-                PtyTaskDebugLog(@"Checking coprocess olnly task %@", coprocessOnlyTask);
-                Coprocess *coprocess = [coprocessOnlyTask coprocess];
-                if (coprocess) {
-                    int fd = [coprocess readFileDescriptor];
-                    if ([handledFds containsObject:@(fd)]) {
-                        NSLog(@"Duplicate fd %d", fd);
-                        continue;
-                    }
-                    [handledFds addObject:@(fd)];
-
-                    [self handleReadOnFileDescriptor:fd task:coprocessOnlyTask withCoprocess:coprocess fdSet:&rfds];
-                    [self handleErrorOnFileDescriptor:fd withCoprocess:coprocess fdSet:&efds];
-
-                    // Handle writes
-                    int coprocessWriteFd = [coprocess writeFileDescriptor];
-                    if ([handledFds containsObject:@(coprocessWriteFd)]) {
-                        NSLog(@"Duplicate fd %d", coprocessWriteFd);
-                        continue;
-                    }
-                    [handledFds addObject:@(coprocessWriteFd)];
-                    [self handleWriteOnFileDescriptor:coprocessWriteFd withCoprocess:coprocess fdSet:&wfds];
-
-                    if ([coprocess eof]) {
-                        [deadpool addObject:@([coprocess pid])];
-                        [coprocess terminate];
-                        [coprocessOnlyTask setCoprocess:nil];
-                        notifyOfCoprocessChange = YES;
-                    }
-                }
-            }
-        }
-
         PtyTaskDebugLog(@"run3: unlock");
         [tasksLock unlock];
         if (notifyOfCoprocessChange) {
@@ -493,8 +459,7 @@ void UnblockTaskNotifier(void) {
 }
 
 // This is run in the main thread.
-- (void)notifyCoprocessChange
-{
+- (void)notifyCoprocessChange {
     [[NSNotificationCenter defaultCenter] postNotificationName:kCoprocessStatusChangeNotification
                                                         object:nil];
 }
