@@ -7,6 +7,8 @@
 
 #import <Foundation/Foundation.h>
 
+#import "iTermMetalConfig.h"
+
 @interface iTermHistogram : NSObject
 
 @property (nonatomic, readonly) NSString *stringValue;
@@ -15,5 +17,10 @@
 
 - (void)addValue:(double)value;
 - (void)mergeFrom:(iTermHistogram *)other;
+- (double)valueAtNTile:(double)ntile;
+- (NSString *)sparklineGraphWithPrecision:(int)precision
+                               multiplier:(double)multiplier
+                                    units:(NSString *)units;
+- (void)clear;
 
 @end
