@@ -9,6 +9,9 @@
 
 NS_ASSUME_NONNULL_BEGIN
 
+@class TmuxController;
+@class VT100RemoteHost;
+
 @protocol iTermComposerTextViewDelegate<NSObject>
 - (void)composerTextViewDidFinishWithCancel:(BOOL)cancel;
 
@@ -22,6 +25,10 @@ NS_ASSUME_NONNULL_BEGIN
 
 @interface iTermStatusBarLargeComposerViewController : NSViewController
 @property (nonatomic, strong) IBOutlet iTermComposerTextView *textView;
+@property (nonatomic, strong, nullable) VT100RemoteHost *host;
+@property (nonatomic, strong, nullable) NSString *workingDirectory;
+@property (nonatomic, copy) NSString *shell;
+@property (nonatomic, weak, nullable) TmuxController *tmuxController;
 
 @end
 
