@@ -22,6 +22,7 @@
 #import "iTermVariableReference.h"
 #import "iTermWarning.h"
 #import "NSArray+iTerm.h"
+#import "NSFileManager+iTerm.h"
 #import "NSJSONSerialization+iTerm.h"
 #import "NSObject+iTerm.h"
 
@@ -331,6 +332,9 @@ static NSString *const iTermStatusBarRPCRegistrationRequestKey = @"registration
     if (-[_dateOfLaunchToFix timeIntervalSinceNow] >= 1) {
         return;
     }
+    if (![[NSFileManager defaultManager] homeDirectoryDotDir]) {
+        return;
+    }
     iTermScriptsMenuController *menuController = [[[iTermApplication sharedApplication] delegate] scriptsMenuController];
     if ([menuController scriptShouldAutoLaunchWithFullPath:_fullPath]) {
         return;
@@ -502,7 +506,8 @@ static NSString *const iTermStatusBarRPCRegistrationRequestKey = @"registration
                                completion:^(id result, NSError *error, NSSet<NSString *> *mutations) {
                                    if (error) {
                                        NSString *message = [NSString stringWithFormat:@"Error in onclick handler: %@\n%@", error.localizedDescription, error.localizedFailureReason];
-                                       [[iTermScriptHistoryEntry globalEntry] addOutput:message];
+                                       [[iTermScriptHistoryEntry globalEntry] addOutput:message
+                                                                             completion:^{}];
                                    }
                                }];
 }
