@@ -5,14 +5,16 @@
 #endif
 
 @class iTermThemeFrame;
+@class NSTitlebarContainerView;
 
 NS_ASSUME_NONNULL_BEGIN
 
 @implementation THE_CLASS
 {
-    int blurFilter;
     double blurRadius_;
-
+    // Hack for a 10.16 issue. Once you set blur from >0 to 0 then it is broken and will never work again.
+    int _minBlur;
+    
     // If set, then windowWillShowInitial is not invoked.
     BOOL _layoutDone;
 
@@ -29,10 +31,17 @@ NS_ASSUME_NONNULL_BEGIN
 
     NSTimeInterval _timeOfLastWindowTitleChange;
     BOOL _needsInvalidateShadow;
+    BOOL _it_restorableStateInvalid;
+    BOOL _validatingMenuItems;
+#if BETA
+    NSString *_lastAlphaChangeStack;
+#endif
+    BOOL _updatingDividerLayer;
 }
 
 @synthesize it_openingSheet;
 @synthesize it_becomingKey;
+@synthesize it_accessibilityResizing;
 
 - (instancetype)initWithContentRect:(NSRect)contentRect
                           styleMask:(NSWindowStyleMask)aStyle
@@ -47,6 +56,18 @@ NS_ASSUME_NONNULL_BEGIN
     if (self) {
         DLog(@"Invalidate cached occlusion: %@ %p", NSStringFromSelector(_cmd), self);
         [[iTermWindowOcclusionChangeMonitor sharedInstance] invalidateCachedOcclusion];
+        [self preventTitlebarDivider];
+    }
+    return self;
+}
+
+- (instancetype)initWithContentRect:(NSRect)contentRect styleMask:(NSWindowStyleMask)style backing:(NSBackingStoreType)backingStoreType defer:(BOOL)flag {
+    self = [super initWithContentRect:contentRect
+                            styleMask:style
+                              backing:backingStoreType
+                                defer:flag];
+    if (self) {
+        [self preventTitlebarDivider];
     }
     return self;
 }
@@ -60,10 +81,56 @@ ITERM_WEAKLY_REFERENCEABLE
         [[iTermWindowOcclusionChangeMonitor sharedInstance] invalidateCachedOcclusion];
     });
     [restoreState_ release];
+#if BETA
+    [_lastAlphaChangeStack release];
+#endif
     [super dealloc];
 
 }
 
+- (void)preventTitlebarDivider {
+    if (@available(macOS 10.16, *)) {
+        static dispatch_once_t onceToken;
+        dispatch_once(&onceToken, ^{
+            static IMP originalImp;
+            originalImp =
+            [iTermSelectorSwizzler permanentlySwizzleSelector:@selector(_updateDividerLayerForController:animated:)
+                                                    fromClass:NSClassFromString(@"NSTitlebarContainerView")
+                                                    withBlock:^(id receiver, id controller, BOOL animated) {
+                void (*f)(id, SEL, id, BOOL) = (void (*)(id, SEL, id, BOOL))originalImp;
+                if (![controller respondsToSelector:@selector(window)]) {
+                    f(receiver, @selector(_updateDividerLayerForController:animated:), controller, animated);
+                    return;
+                }
+                NSWindow<PTYWindow> *window = (NSWindow<PTYWindow> *)[controller window];
+                if (![window conformsToProtocol:@protocol(PTYWindow)]) {
+                    f(receiver, @selector(_updateDividerLayerForController:animated:), controller, animated);
+                    return;
+                }
+
+                [window setUpdatingDividerLayer:YES];
+                f(receiver, @selector(_updateDividerLayerForController:animated:), controller, animated);
+                [window setUpdatingDividerLayer:NO];
+            }];
+        });
+    }
+}
+
+- (NSTitlebarSeparatorStyle)titlebarSeparatorStyle NS_AVAILABLE_MAC(10_16) {
+    if (_updatingDividerLayer) {
+        id<PTYWindow> ptywindow = (id<PTYWindow>)self;
+        if ([ptywindow.ptyDelegate terminalWindowShouldHaveTitlebarSeparator]) {
+            return NSTitlebarSeparatorStyleShadow;
+        }
+        return NSTitlebarSeparatorStyleNone;
+    }
+    return [super titlebarSeparatorStyle];
+}
+
+- (void)setUpdatingDividerLayer:(BOOL)value {
+    _updatingDividerLayer = value;
+}
+
 - (void)setDocumentEdited:(BOOL)documentEdited {
     [super setDocumentEdited:documentEdited];
     [[NSNotificationCenter defaultCenter] postNotificationName:iTermWindowDocumentedEditedDidChange object:self];
@@ -89,7 +156,16 @@ ITERM_WEAKLY_REFERENCEABLE
             return ![self.ptyDelegate anyFullScreen];
         }
     } else {
-        return [super validateMenuItem:item];
+        _validatingMenuItems = YES;
+        const BOOL result = [super validateMenuItem:item];
+        _validatingMenuItems = NO;
+        return result;
+    }
+}
+
+- (void)performWindowDragWithEvent:(NSEvent *)event {
+    if ([self.ptyDelegate ptyWindowIsDraggable:self]) {
+        [super performWindowDragWithEvent:event];
     }
 }
 
@@ -107,7 +183,13 @@ ITERM_WEAKLY_REFERENCEABLE
 }
 
 - (NSString *)description {
-    return [NSString stringWithFormat:@"<%@: %p frame=%@ title=%@ alpha=%f isMain=%d isKey=%d isVisible=%d delegate=%p>",
+    NSString *extra = @"";
+#if BETA
+    if (_lastAlphaChangeStack) {
+        extra = [NSString stringWithFormat:@" Alpha last changed from:\n%@\n", _lastAlphaChangeStack];
+    }
+#endif
+    return [NSString stringWithFormat:@"<%@: %p frame=%@ title=%@ alpha=%f isMain=%d isKey=%d isVisible=%d delegate=%p%@>",
             [self class],
             self,
             [NSValue valueWithRect:self.frame],
@@ -116,7 +198,8 @@ ITERM_WEAKLY_REFERENCEABLE
             (int)self.isMainWindow,
             (int)self.isKeyWindow,
             (int)self.isVisible,
-            self.delegate];
+            self.delegate,
+            extra];
 }
 
 - (NSString *)windowIdentifier {
@@ -127,7 +210,26 @@ ITERM_WEAKLY_REFERENCEABLE
     return [NSString stringWithFormat:@"window-%ld", (long)_uniqueNumber];
 }
 
+- (void)setIt_restorableStateInvalid:(BOOL)it_restorableStateInvalid {
+    _it_restorableStateInvalid = NO;
+}
+
+- (BOOL)it_restorableStateInvalid {
+    return _it_restorableStateInvalid;
+}
+
+- (void)invalidateRestorableState {
+    self.it_restorableStateInvalid = YES;
+    [super invalidateRestorableState];
+}
+
+- (void)encodeRestorableStateWithCoder:(NSCoder *)coder backgroundQueue:(NSOperationQueue *)queue {
+    self.it_restorableStateInvalid = NO;
+    [super encodeRestorableStateWithCoder:coder backgroundQueue:queue];
+}
+
 - (void)encodeRestorableStateWithCoder:(NSCoder *)coder {
+    self.it_restorableStateInvalid = NO;
     [super encodeRestorableStateWithCoder:coder];
     [coder encodeObject:restoreState_ forKey:kTerminalWindowStateRestorationWindowArrangementKey];
 }
@@ -138,18 +240,19 @@ ITERM_WEAKLY_REFERENCEABLE
 }
 
 - (void)enableBlur:(double)radius {
-    const double kEpsilon = 0.001;
-    if (blurFilter && fabs(blurRadius_ - radius) < kEpsilon) {
-        return;
-    }
-
-    CGSConnectionID con = CGSMainConnectionID();
+    CGSConnectionID con = CGSDefaultConnectionForThread();
     if (!con) {
         return;
     }
     CGSSetWindowBackgroundBlurRadiusFunction* function = GetCGSSetWindowBackgroundBlurRadiusFunction();
     if (function) {
-        function(con, [self windowNumber], (int)radius);
+        if (@available(macOS 10.16, *)) {
+            if (radius >= 1) {
+                _minBlur = 1;
+            }
+        }
+        DLog(@"enable blur with radius %@ for window %@", @(MAX(_minBlur, radius)), self);
+        function(con, [self windowNumber], (int)MAX(_minBlur, radius));
     } else {
         NSLog(@"Couldn't get blur function");
     }
@@ -157,18 +260,15 @@ ITERM_WEAKLY_REFERENCEABLE
 }
 
 - (void)disableBlur {
-    CGSConnectionID con = CGSMainConnectionID();
+    CGSConnectionID con = CGSDefaultConnectionForThread();
     if (!con) {
         return;
     }
 
     CGSSetWindowBackgroundBlurRadiusFunction* function = GetCGSSetWindowBackgroundBlurRadiusFunction();
     if (function) {
-        function(con, [self windowNumber], 0);
-    } else if (blurFilter) {
-        CGSRemoveWindowFilter(con, (CGSWindowID)[self windowNumber], blurFilter);
-        CGSReleaseCIFilter(CGSMainConnectionID(), blurFilter);
-        blurFilter = 0;
+        DLog(@"disable blur for window %@", self);
+        function(con, [self windowNumber], MAX(_minBlur, 0));
     }
 }
 
@@ -278,6 +378,7 @@ ITERM_WEAKLY_REFERENCEABLE
     [[iTermWindowOcclusionChangeMonitor sharedInstance] invalidateCachedOcclusion];
     self.it_becomingKey = YES;
     [super makeKeyAndOrderFront:sender];
+    [self.ptyDelegate ptyWindowDidMakeKeyAndOrderFront:self];
     self.it_becomingKey = NO;
 }
 
@@ -331,6 +432,14 @@ ITERM_WEAKLY_REFERENCEABLE
 - (void)setAlphaValue:(CGFloat)alphaValue {
     DLog(@"Invalidate cached occlusion: %@ %p", NSStringFromSelector(_cmd), self);
     [[iTermWindowOcclusionChangeMonitor sharedInstance] invalidateCachedOcclusion];
+#if BETA
+    if (alphaValue < 0.01 && !_lastAlphaChangeStack) {
+        _lastAlphaChangeStack = [[[NSThread callStackSymbols] componentsJoinedByString:@"\n"] copy];
+    } else if (alphaValue > 0.01) {
+        [_lastAlphaChangeStack release];
+        _lastAlphaChangeStack = nil;
+    }
+#endif
     [super setAlphaValue:alphaValue];
 }
 
@@ -441,9 +550,20 @@ ITERM_WEAKLY_REFERENCEABLE
     return NSWindowTabbingModeDisallowed;
 }
 
+- (void)_moveToScreen:(id)sender {
+    if (![[THE_CLASS superclass] instancesRespondToSelector:_cmd]) {
+        return;
+    }
+    if ([sender isKindOfClass:[NSScreen class]]) {
+        [self.ptyDelegate terminalWindowWillMoveToScreen:sender];
+    }
+    [super _moveToScreen:sender];
+}
+
 - (void)setFrame:(NSRect)frameRect display:(BOOL)flag {
-    DLog(@"setFrame:%@ display:%@ maxy=%@ from\n%@",
+    DLog(@"setFrame:%@ display:%@ maxy=%@ of %@ from\n%@",
          NSStringFromRect(frameRect), @(flag), @(NSMaxY(frameRect)),
+          self.delegate,
          [NSThread callStackSymbols]);
     [super setFrame:frameRect display:flag];
 }
@@ -478,6 +598,12 @@ ITERM_WEAKLY_REFERENCEABLE
 }
 #endif
 
+- (void)accessibilitySetSizeAttribute:(id)arg1 {
+    self.it_accessibilityResizing += 1;
+    [super accessibilitySetSizeAttribute:arg1];
+    self.it_accessibilityResizing -= 1;
+}
+
 - (NSColor *)it_terminalWindowDecorationBackgroundColor {
     return [self.ptyDelegate terminalWindowDecorationBackgroundColor];
 }
@@ -512,6 +638,10 @@ ITERM_WEAKLY_REFERENCEABLE
     });
 }
 
+- (BOOL)isMovable {
+    return _validatingMenuItems || [super isMovable];
+}
+
 NS_ASSUME_NONNULL_END
 
 @end
